#!/bin/sh

# NOTE: There should be a stable user identifier defined, not that of the user
# NOTE: running this script.

PROGNAME=$(basename "$0")

if [ "$1" = '-q' ] ; then
    QUIET=$1
    shift 1
else
    QUIET=
fi

PKGDIR=$(realpath "$1")
E2ACCESS_DIR=$(realpath "$2")
TARGET=$(realpath "$3")

if [ ! -e "$PKGDIR" ] || [ ! -e "$E2ACCESS_DIR" ] || [ ! "$TARGET" ] ; then
    cat 1>&2 <<EOF
Usage: $PROGNAME [ -q ] <package directory> <e2access directory> <target>

Package directory: $PKGDIR
e2access directory: $E2ACCESS_DIR
Target filesystem: $TARGET
EOF
    exit 1
fi

# Initialise the program details and environment.

E2ACCESS="$E2ACCESS_DIR"/e2access
export LD_LIBRARY_PATH="$E2ACCESS_DIR"

# Make a filesystem for the example.

mkdir tmp_e2test
cd tmp_e2test

# Set a sensible umask.

umask 0022

mkdir -p home/paulb/public
echo "Public!" > home/paulb/public/message

# Set a restrictive umask.

umask 0077

echo "Not public!" > home/paulb/public/message2

mkdir -p home/paulb/private
echo "Private!" > home/paulb/private/message

# Set a permissive umask.

umask 0000

mkdir -p home/paulb/shared
echo "Editable!" > home/paulb/shared/message

# Restore the umask for the remaining files.

umask 0022

mkdir -p home/paulb/many
cd home/paulb/many

# Populate the directory with plenty of files.

for N in `seq 1 400`; do
    echo "Contents of #$N." > "file-$N".txt
done

cd ..

# Put a file in the directory above.

cp "$PKGDIR/../docs/LICENCE.txt" .

# Leave the filesystem root.

cd ../..

# Create a filesystem image.

EXTRA=2000
SIZE=$(du -s -k home | cut -f 1)
TOTAL=$(($SIZE + $EXTRA))

dd if=/dev/zero of="$TARGET" bs=1024 count=$TOTAL

if ! $(/sbin/mkfs.ext2 -q "$TARGET") ; then
    exit 1
fi

# Add the directories and files to the image.

for DIR in home/paulb/private home/paulb/public home/paulb/shared home/paulb/many home/paulb ; do
    "$E2ACCESS" "$TARGET" --make-dirs "$DIR"
    "$E2ACCESS" "$TARGET" --copy-in $(find "$DIR" -maxdepth 1 -type f | sort) "$DIR"
done

# Leave the root of the filesystem.

cd ..

if [ ! "$QUIET" ] ; then
    "$E2ACCESS" "$TARGET" --list-dirs ''
    "$E2ACCESS" "$TARGET" --list-dirs 'home'
    "$E2ACCESS" "$TARGET" --list-dirs 'home/paulb'
    "$E2ACCESS" "$TARGET" --list-dirs 'home/paulb/many'
    "$E2ACCESS" "$TARGET" --list-dirs 'home/paulb/private'
    "$E2ACCESS" "$TARGET" --list-dirs 'home/paulb/public'
    "$E2ACCESS" "$TARGET" --list-dirs 'home/paulb/shared'
fi

rm -r tmp_e2test

# vim: tabstop=4 expandtab shiftwidth=4
